import json
from dataclasses import dataclass, asdict

from .util import JsonEncoder
from yankee.util import is_valid

@dataclass
class Row():
    def to_dict(self):
        return asdict(self, dict_factory=lambda i: dict([(k, v) for k, v in i if is_valid(v)]))

    def to_pandas(self):
        """Convert object to Pandas Series"""
        import pandas as pd

        return pd.Series(self.to_dict())

    def to_json(self, *args, **kwargs):
        return json.dumps(self.to_dict(), *args, cls=JsonEncoder, **kwargs)


from yankee.base import schema

from .mixin import JsonMixin


class Schema(JsonMixin, schema.Schema):
    pass

class PolymorphicSchema(JsonMixin, schema.PolymorphicSchema):
    pass

class RegexSchema(JsonMixin, schema.RegexSchema):
    pass

class ZipSchema(JsonMixin, schema.ZipSchema):
    pass


import lxml.etree as ET

from .accessor import xpath_accessor

class XmlMixin(object):
    class Meta:
        accessor_function = xpath_accessor
        infer_keys = False

    def to_string(self, elem):
        return elem if isinstance(elem, str) else elem.text or ""

    def convert_groupdict(self, dictionary):
        root = ET.Element("root")
        for k, v in dictionary.items():
            subel = ET.SubElement(root, k)
            subel.text = v
        return root

    @property
    def raw_text(self):
        return ET.tostring(self.raw, pretty_print=True).decode()
